//
//  KPDatasource.m
//  TableViewPlayground
//
//  Created by Kalle Persson on 05/10/13.
//  Copyright (c) 2013 Kalle Persson. All rights reserved.
//

#import "KPDatasource.h"
#import "KPCell.h"
#import "KPCellAccessoryView.h"
#import "KPItem.h"

NSString * const kUserDefaultsItemsKey = @"items";
NSString * const kReuseIdentifier = @"cell";

@interface KPDatasource()

@property (nonatomic) KPItemCategory category;
@property (nonatomic, strong) NSMutableArray *visibleItems;
@property (nonatomic, strong) NSMutableArray *hiddenItems;

@end

@implementation KPDatasource

- (id)init {
    if (!(self == [super init]))
        return nil;
    
    _category = KPItemCategoryInbox;
    
    return self;
}

- (id)initWithCategory:(KPItemCategory)category
{
    if (![self init])
        return nil;
    
    _category = category;
    
    [self loadData];
    
    return self;
}

- (void)loadData
{
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    
    NSData *encodedItems = [userDefaults objectForKey:kUserDefaultsItemsKey];
    NSArray *items = [NSKeyedUnarchiver unarchiveObjectWithData:encodedItems];
    if(!items)
        items = @[];
    
    NSString *visiblePredicateFormat = @"";
    NSString *hiddenPredicateFormat = @"";
    
    switch (_category) {
        case KPItemCategoryInbox:
            visiblePredicateFormat = @"isArchived == NO";
            hiddenPredicateFormat = @"isArchived == YES";
        break;
        case KPItemCategoryArchived:
            visiblePredicateFormat = @"isArchived == YES";
            hiddenPredicateFormat = @"isArchived == NO";
        break;
        case KPItemCategoryFavourited:
            visiblePredicateFormat = @"isFavourited == YES";
            hiddenPredicateFormat = @"isFavourited == NO";
        break;
        case KPItemCategoryCategoryCount:
        break;
    }
    
    self.visibleItems = [[items filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:visiblePredicateFormat]] mutableCopy];
    self.hiddenItems = [[items filteredArrayUsingPredicate:[NSPredicate predicateWithFormat:hiddenPredicateFormat]] mutableCopy];
}

#pragma mark - UITableViewDataSource methods

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView actualNumberOfRowsInSection:(NSInteger)section
{
    return [self.visibleItems count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.visibleItems count];
//    NSInteger actualAmountOfRows = [self tableView:tableView actualNumberOfRowsInSection:section];
//    return actualAmountOfRows > 0 ? self.items.count + 1 : 0;
}

- (KPCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    KPCell *cell = [tableView dequeueReusableCellWithIdentifier:kReuseIdentifier];
    if (!cell) {
        cell = [[KPCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kReuseIdentifier];
    }
    
    KPItem *item = self.visibleItems[indexPath.row];
    
    cell.backgroundColor = [UIColor whiteColor];
    
    UITextField *textField = [[UITextField alloc] initWithFrame:CGRectZero];
    if (indexPath.row < self.visibleItems.count) {
        
        KPCellAccessoryView *leftAccessoryView = [[KPCellAccessoryView alloc] initWithFrame:CGRectZero];
        leftAccessoryView.text = self.category == KPItemCategoryArchived ? @"Unarchive" : @"Archive";
        leftAccessoryView.backgroundColor  = [UIColor colorWithRed:227/255.0 green:125/255.0 blue:111/225.0 alpha:1.0];
        cell.leftAccessoryView = leftAccessoryView;

        if (self.category != KPItemCategoryArchived) {
            KPCellAccessoryView *rightAccessoryView = [[KPCellAccessoryView alloc] initWithFrame:CGRectZero];
            rightAccessoryView.backgroundColor  = [UIColor colorWithRed:179/255.0 green:227/255.0 blue:111/225.0 alpha:1.0];
            rightAccessoryView.inactiveText = @"☆";
            rightAccessoryView.activeText = @"★";
            rightAccessoryView.font = [UIFont systemFontOfSize:24.0];
            cell.rightAccessoryView = rightAccessoryView;
        }
        
        
        textField.text = item.title;
        
        if (item.isFavourited) {
            textField.textColor = [UIColor whiteColor];
            cell.backgroundColor = [UIColor colorWithRed:179/255.0 green:227/255.0 blue:111/225.0 alpha:1.0];
        }
        
    } else {
        textField.text = @"Archive all items";
        textField.textColor = [UIColor colorWithWhite:0.6 alpha:1];
        textField.textAlignment = NSTextAlignmentCenter;
        textField.enabled = NO;
    }
    cell.textField = textField;
    
    // NOTE: Not sure at all that this is a good idea
    cell.textField.delegate = self.delegate;
    cell.delegate = self.delegate;
    
    return cell;
}

- (void)toggleArchiveRowAtIndexPath:(NSIndexPath *)indexPath
{
    KPItem *itemToBeToggled = [self.visibleItems objectAtIndex:indexPath.row];
    if (itemToBeToggled.isArchived) {
        [itemToBeToggled unFavourite];
    } else {
        [itemToBeToggled archive];
    }
    
    [self.hiddenItems addObject:itemToBeToggled];
    [self.visibleItems removeObject:itemToBeToggled];
    
    [self synchronizeData];
}

- (void)toggleFavouriteRowAtIndexPath:(NSIndexPath *)indexPath
{
    KPItem *itemToBeToggled = [self.visibleItems objectAtIndex:indexPath.row];
    if (itemToBeToggled.isFavourited) {
        [itemToBeToggled unFavourite];
        if (self.category == KPItemCategoryFavourited) {
            [self.hiddenItems addObject:itemToBeToggled];
            [self.visibleItems removeObject:itemToBeToggled];
        }
    } else {
        [itemToBeToggled favourite];
    }
    

    [self synchronizeData];
}

- (void)insertRowAtIndex:(NSUInteger)index
{
    [self.visibleItems insertObject:[KPItem itemWithTitle:@""] atIndex:index];

    [self synchronizeData];
}

- (void)updateText:(NSString *)text forRowAtIndexPath:(NSIndexPath *)indexPath
{
    KPItem *updatedItem = [self.visibleItems objectAtIndex:indexPath.row];
    updatedItem.title = text;
    //[self.items setObject:text atIndexedSubscript:indexPath.row];
    [self synchronizeData];
}

- (void)synchronizeData
{
    NSMutableArray *items = [NSMutableArray new];
    [items addObjectsFromArray:self.visibleItems];
    [items addObjectsFromArray:self.hiddenItems];
    
    NSData *encodedObject = [NSKeyedArchiver archivedDataWithRootObject:items];
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:encodedObject forKey:kUserDefaultsItemsKey];
    NSLog(@"Archived %@", items);
}

- (BOOL)tableView:(UITableView *)tableView canSwipeRowAtIndexPath:(NSIndexPath *)indexPath
{
    return indexPath.row < self.visibleItems.count;
}

- (void)archiveAllRows {
    [self.visibleItems makeObjectsPerformSelector:@selector(archive)];
    [self synchronizeData];
}

- (NSArray *)indexPathsForAllRowsInSection:(NSUInteger)section includeDeleteRow:(BOOL)includeDeleteRow {
    NSMutableArray *indexPaths = [NSMutableArray new];
    for(NSInteger i = 0; i < self.visibleItems.count + (includeDeleteRow ? 1 : 0); i++)
        [indexPaths addObject:[NSIndexPath indexPathForRow:i inSection:section]];
    return indexPaths;
}

@end
